@@ -54,13 +54,6 @@ enum
 
 static const char* CONTENTPROTECTION_TAG = "ContentProtection";
 
-static const char* ltranslate(const char* in)
-{
-  if (strlen(in) == 2 || strlen(in) == 3 || (strlen(in) > 3 && in[2] == '-'))
-    return in;
-  return "unk";
-}
-
 DASHTree::DASHTree()
 {
 }
@@ -79,7 +72,8 @@ static uint8_t GetChannels(const char** attr)
   }
   if (schemeIdUri && value)
   {
-    if (strcmp(schemeIdUri, "urn:mpeg:dash:23003:3:audio_channel_configuration:2011") == 0)
+    if (strcmp(schemeIdUri, "urn:mpeg:dash:23003:3:audio_channel_configuration:2011") == 0 ||
+             strcmp(schemeIdUri, "urn:mpeg:mpegB:cicp:ChannelConfiguration") == 0)
       return atoi(value);
     else if (strcmp(schemeIdUri, "urn:dolby:dash:audio_channel_configuration:2011") == 0 ||
              strcmp(schemeIdUri, "tag:dolby.com,2014:dash:audio_channel_configuration:2011") == 0)
@@ -107,8 +101,6 @@ static unsigned int ParseSegmentTemplate(const char** attr,
       tpl.duration = atoi((const char*)*(attr + 1));
     else if (strcmp((const char*)*attr, "media") == 0)
       tpl.media = (const char*)*(attr + 1);
-    else if (strcmp((const char*)*attr, "presentationTimeOffset") == 0)
-      tpl.presentationTimeOffset = atoll((const char*)*(attr + 1));
     else if (strcmp((const char*)*attr, "startNumber") == 0)
       startNumber = atoi((const char*)*(attr + 1));
     else if (strcmp((const char*)*attr, "initialization") == 0)
@@ -123,6 +115,8 @@ static unsigned int ParseSegmentTemplate(const char** attr,
   {
     if (!tpl.media.empty() && tpl.media[0] == '/')
       tpl.media = baseDomain + tpl.media;
+    else if (!baseURL.empty() && baseURL.back() != '/')
+      tpl.media = baseURL + '/' + tpl.media;
     else
       tpl.media = baseURL + tpl.media;
   }
@@ -132,6 +126,8 @@ static unsigned int ParseSegmentTemplate(const char** attr,
   {
     if (!tpl.initialization.empty() && tpl.initialization[0] == '/')
       tpl.initialization = baseDomain + tpl.initialization;
+    else if (!baseURL.empty() && baseURL.back() != '/')
+      tpl.initialization = baseURL + '/' + tpl.initialization;
     else
       tpl.initialization = baseURL + tpl.initialization;
   }
@@ -208,7 +204,7 @@ bool ParseContentProtection(const char** attr, DASHTree* dash)
       else
         urnFound = stricmp(dash->supportedKeySystem_.c_str(), (const char*)*(attr + 1)) == 0;
     }
-    else if (strcmp((const char*)*attr, "cenc:default_KID") == 0)
+    else if (endswith((const char*)*attr, "default_KID"))
       defaultKID = (const char*)*(attr + 1);
     attr += 2;
   }
@@ -403,7 +399,7 @@ static void XMLCALL start(void* data, const char* el, const char** attr)
           }
           else if (dash->currentNode_ & MPDNODE_CONTENTPROTECTION)
           {
-            if (strcmp(el, "cenc:pssh") == 0)
+            if (endswith(el, "pssh"))
               dash->currentNode_ |= MPDNODE_PSSH;
             else if (strcmp(el, "widevine:license") == 0)
             {
@@ -559,6 +555,9 @@ static void XMLCALL start(void* data, const char* el, const char** attr)
           {
             dash->currentNode_ |= MPDNODE_SEGMENTTIMELINE;
             dash->adp_timelined_ = true;
+
+            if (dash->update_parameter_.empty() && dash->has_timeshift_buffer_)
+              dash->update_parameter_ = "full";
           }
         }
         else if (dash->currentNode_ & MPDNODE_SEGMENTDURATIONS)
@@ -569,7 +568,7 @@ static void XMLCALL start(void* data, const char* el, const char** attr)
         }
         else if (dash->currentNode_ & MPDNODE_CONTENTPROTECTION)
         {
-          if (strcmp(el, "cenc:pssh") == 0)
+          if (endswith(el, "pssh"))
             dash->currentNode_ |= MPDNODE_PSSH;
           else if (strcmp(el, "widevine:license") == 0)
           {
@@ -663,6 +662,7 @@ static void XMLCALL start(void* data, const char* el, const char** attr)
           dash->current_representation_->width_ = dash->adpwidth_;
           dash->current_representation_->height_ = dash->adpheight_;
           dash->current_representation_->fpsRate_ = dash->adpfpsRate_;
+          dash->current_representation_->fpsScale_ = dash->adpfpsScale_;
           dash->current_representation_->aspect_ = dash->adpaspect_;
           dash->current_representation_->containerType_ = dash->adpContainerType_;
 
@@ -685,9 +685,12 @@ static void XMLCALL start(void* data, const char* el, const char** attr)
               dash->current_representation_->samplingRate_ =
                   static_cast<uint32_t>(atoi((const char*)*(attr + 1)));
             else if (strcmp((const char*)*attr, "frameRate") == 0)
+            {
+              dash->current_representation_->fpsScale_ = 1;
               sscanf((const char*)*(attr + 1), "%" SCNu32 "/%" SCNu32,
                      &dash->current_representation_->fpsRate_,
                      &dash->current_representation_->fpsScale_);
+            }
             else if (strcmp((const char*)*attr, "id") == 0)
               dash->current_representation_->id = (const char*)*(attr + 1);
             else if (strcmp((const char*)*attr, "codecPrivateData") == 0)
@@ -707,7 +710,8 @@ static void XMLCALL start(void* data, const char* el, const char** attr)
                 else if (strncmp(dash->current_adaptationset_->mimeType_.c_str(), "audio", 5) == 0)
                   dash->current_adaptationset_->type_ = DASHTree::AUDIO;
                 else if (strncmp(dash->current_adaptationset_->mimeType_.c_str(), "application",
-                                 11) == 0)
+                                 11) == 0 ||
+                         strncmp(dash->current_adaptationset_->mimeType_.c_str(), "text", 4) == 0)
                   dash->current_adaptationset_->type_ = DASHTree::SUBTITLE;
               }
               if (strstr(dash->current_adaptationset_->mimeType_.c_str(), "/webm"))
@@ -719,6 +723,28 @@ static void XMLCALL start(void* data, const char* el, const char** attr)
             attr += 2;
           }
 
+          if (dash->current_representation_->codecs_.empty())
+          {
+            if (dash->current_adaptationset_->mimeType_ == "text/vtt")
+              dash->current_representation_->codecs_ = "wvtt";
+            else if (dash->current_adaptationset_->mimeType_ == "application/ttml+xml")
+              dash->current_representation_->codecs_ = "ttml";
+          }
+
+          if (dash->current_adaptationset_->type_ != DASHTree::SUBTITLE)
+          {
+            if (dash->current_representation_->codecs_ == "wvtt")
+            {
+              dash->current_adaptationset_->type_ = DASHTree::SUBTITLE;
+              dash->current_adaptationset_->mimeType_ = "text/vtt";
+            }
+            else if (dash->current_representation_->codecs_ == "ttml")
+            {
+              dash->current_adaptationset_->type_ = DASHTree::SUBTITLE;
+              dash->current_adaptationset_->mimeType_ = "application/ttml+xml";
+            }
+          }
+
           if (dash->current_adaptationset_->type_ == DASHTree::SUBTITLE &&
               (dash->current_adaptationset_->mimeType_ == "application/ttml+xml" ||
                dash->current_adaptationset_->mimeType_ == "text/vtt"))
@@ -729,14 +755,6 @@ static void XMLCALL start(void* data, const char* el, const char** attr)
               dash->current_representation_->containerType_ = AdaptiveTree::CONTAINERTYPE_TEXT;
           }
 
-          if (dash->current_adaptationset_->type_ != DASHTree::SUBTITLE &&
-              dash->current_representation_->codecs_ == "wvtt")
-            dash->current_adaptationset_->type_ = DASHTree::SUBTITLE;
-
-          if (dash->current_adaptationset_->mimeType_ == "text/vtt" &&
-              dash->current_representation_->codecs_.empty())
-            dash->current_representation_->codecs_ = "wvtt";
-
           dash->current_representation_->segtpl_ = dash->current_adaptationset_->segtpl_;
           if (!dash->current_adaptationset_->segtpl_.media.empty())
           {
@@ -854,6 +872,7 @@ static void XMLCALL start(void* data, const char* el, const char** attr)
         dash->adpwidth_ = 0;
         dash->adpheight_ = 0;
         dash->adpfpsRate_ = 0;
+        dash->adpfpsScale_ = 1;
         dash->adpaspect_ = 0.0f;
         dash->adp_pssh_set_ = 0;
         dash->adpContainerType_ = AdaptiveTree::CONTAINERTYPE_MP4;
@@ -882,7 +901,7 @@ static void XMLCALL start(void* data, const char* el, const char** attr)
           else if (strcmp((const char*)*attr, "group") == 0)
             dash->current_adaptationset_->group_ = (const char*)*(attr + 1);
           else if (strcmp((const char*)*attr, "lang") == 0)
-            dash->current_adaptationset_->language_ = ltranslate((const char*)*(attr + 1));
+            dash->current_adaptationset_->language_ = (const char*)*(attr + 1);
           else if (strcmp((const char*)*attr, "mimeType") == 0)
             dash->current_adaptationset_->mimeType_ = (const char*)*(attr + 1);
           else if (strcmp((const char*)*attr, "name") == 0)
@@ -894,7 +913,9 @@ static void XMLCALL start(void* data, const char* el, const char** attr)
           else if (strcmp((const char*)*attr, "height") == 0)
             dash->adpheight_ = static_cast<uint16_t>(atoi((const char*)*(attr + 1)));
           else if (strcmp((const char*)*attr, "frameRate") == 0)
-            dash->adpfpsRate_ = static_cast<uint32_t>(atoi((const char*)*(attr + 1)));
+            sscanf((const char*)*(attr + 1), "%" SCNu32 "/%" SCNu32,
+                    &dash->adpfpsRate_,
+                    &dash->adpfpsScale_);
           else if (strcmp((const char*)*attr, "par") == 0)
           {
             int w, h;
@@ -1002,12 +1023,11 @@ static void XMLCALL start(void* data, const char* el, const char** attr)
   else if (strcmp(el, "MPD") == 0)
   {
     const char *mpt(0), *tsbd(0);
-    bool bStatic(false);
 
     dash->firstStartNumber_ = 0;
 
     dash->overallSeconds_ = 0;
-    dash->stream_start_ = time(0);
+    dash->stream_start_ = dash->GetNowTime();
     dash->mpd_url_ = dash->base_url_;
 
     for (; *attr;)
@@ -1015,11 +1035,11 @@ static void XMLCALL start(void* data, const char* el, const char** attr)
       if (strcmp((const char*)*attr, "mediaPresentationDuration") == 0)
       {
         mpt = (const char*)*(attr + 1);
-        bStatic = true;
       }
-      else if (strcmp((const char*)*attr, "type") == 0)
+      else if (strcmp((const char*)*attr, "type") == 0 &&
+               strcmp((const char*)*(attr + 1), "dynamic") == 0)
       {
-        bStatic = strcmp((const char*)*(attr + 1), "static") == 0;
+        dash->has_timeshift_buffer_ = true;
       }
       else if (strcmp((const char*)*attr, "timeShiftBufferDepth") == 0)
       {
@@ -1028,10 +1048,6 @@ static void XMLCALL start(void* data, const char* el, const char** attr)
       }
       else if (strcmp((const char*)*attr, "availabilityStartTime") == 0)
         dash->available_time_ = getTime((const char*)*(attr + 1));
-      else if (strcmp((const char*)*attr, "publishTime") == 0)
-        dash->publish_time_ = getTime((const char*)*(attr + 1));
-      else if (strcmp((const char*)*attr, "testTime") == 0)
-        dash->stream_start_ = getTime((const char*)*(attr + 1));
       else if (strcmp((const char*)*attr, "minimumUpdatePeriod") == 0)
       {
         uint64_t dur(0);
@@ -1043,15 +1059,11 @@ static void XMLCALL start(void* data, const char* el, const char** attr)
 
     if (!mpt)
       mpt = tsbd;
-    dash->has_timeshift_buffer_ = !bStatic;
 
     AddDuration(mpt, dash->overallSeconds_, 1);
     dash->has_overall_seconds_ = dash->overallSeconds_ > 0;
 
     uint64_t overallsecs(dash->overallSeconds_ ? dash->overallSeconds_ + 60 : 86400);
-    if (!dash->base_time_ && dash->publish_time_ && dash->available_time_ &&
-        dash->publish_time_ - dash->available_time_ > overallsecs)
-      dash->base_time_ = dash->publish_time_ - dash->available_time_ - overallsecs;
     dash->minPresentationOffset = ~0ULL;
 
     dash->currentNode_ |= MPDNODE_MPD;
@@ -1149,7 +1161,7 @@ static void XMLCALL end(void* data, const char* el)
           {
             if (dash->currentNode_ & MPDNODE_PSSH)
             {
-              if (strcmp(el, "cenc:pssh") == 0)
+              if (endswith(el, "pssh"))
               {
                 dash->current_pssh_ = dash->strXMLText_;
                 dash->currentNode_ &= ~MPDNODE_PSSH;
@@ -1240,14 +1252,9 @@ static void XMLCALL end(void* data, const char* el)
                   if (!timeBased && dash->has_timeshift_buffer_ &&
                       tpl.duration)
                   {
-                    // get the closest time to calculate start_number
-                    uint64_t calc_time =
-                        dash->publish_time_ ? dash->publish_time_ : dash->stream_start_;
-                    uint64_t sample_time = tpl.presentationTimeOffset
-                                               ? tpl.presentationTimeOffset / tpl.timescale
-                                               : dash->current_period_->start_ /  1000;
+                    uint64_t sample_time = dash->current_period_->start_ /  1000;
 
-                    seg.range_end_ += (static_cast<int64_t>(calc_time - dash->available_time_ -
+                    seg.range_end_ += (static_cast<int64_t>(dash->stream_start_ - dash->available_time_ -
                                                             overallSeconds - sample_time)) *
                                           tpl.timescale / tpl.duration +
                                       1;
@@ -1348,7 +1355,7 @@ static void XMLCALL end(void* data, const char* el)
         {
           if (dash->currentNode_ & MPDNODE_PSSH)
           {
-            if (strcmp(el, "cenc:pssh") == 0)
+            if (endswith(el, "pssh"))
             {
               dash->current_pssh_ = dash->strXMLText_;
               dash->currentNode_ &= ~MPDNODE_PSSH;
